# -*- coding: utf-8 -*-
"""
Created on Sat Jan 11 14:57:41 2020

@author: mendez
"""

# Exercise 1 in Chap 7

# Test The Convolution theorem and Topelitz Matrices

import numpy as np
from scipy import signal
import matplotlib.pyplot as plt  

# Run this script line by line !

## Step 0: Construct the signal with its scales:
n_t=4096 # Number of point
k=np.arange(0,n_t,1)
f_s=1000; dt=1/f_s # Compute Delta t
t_k=k*dt # Time domain axis
# Mode 1
u_1=2*np.sin(2*np.pi*20*t_k)*np.exp(-(t_k-1)**2/(0.05))\
# Mode 2
u_2=1*np.sin(2*np.pi*90*t_k)*np.exp(-(t_k-2.2)**2/(0.05))\
# Mode 3
u_3=1*np.sin(2*np.pi*1*t_k)
u=u_1+u_2+u_3+np.random.normal(0, 0.2, size=n_t) 


## Part 1:Compute the (normalized) DFT via Numpy's fft
U=np.fft.fft(u,n_t)/np.sqrt(n_t)
## Part 2: Obtain the previous result from Matrix multiplication
# Step 1: Prepare Fourier Matrix 
Psi_F=np.conj(np.fft.fft(np.eye(n_t))/np.sqrt(n_t))
# Step 2: Matrix multiplication for the DFT:
U_DFT=np.dot(np.conj(Psi_F),u)
# Compare real and imaginary parts:
plt.subplot(2,1,1)
plt.plot(np.angle(U),np.angle(U_DFT))
plt.title('Compare Absolute Value')
plt.tight_layout
plt.subplot(2,1,2)
plt.plot(np.abs(U),np.abs(U_DFT))
plt.title('Compare Argument')
plt.tight_layout
plt.close(fig='all')

# Consider a moving average kernel
N=211
h_LP=signal.firwin(N, 20, window='hamming', fs=f_s)
# Extend h to length of n_t
h_L_c=np.zeros(u.shape);h_L_c[0:N]=h_LP 

# Import function for circulant matrix
from scipy.linalg import circulant
# Construct circulant matrix
C_H=circulant(h_L_c)
# Filter Signal in time domain
y_Filt=np.dot(C_H,u)



###### Compute the filtered signal via fft#####
# Compute frequency transfer function
H_L=np.fft.fft(h_LP,len(t_k))
# Apply filter in frequency domain and invert
y_Filt2 = np.real(np.fft.ifft(H_L*U))*np.sqrt(n_t)
###### Compute the filtered signal via eq 25 #####
# Construct Eigenvalue matrix:
H_d=np.diag(H_L)
# Construct Fourier Matrix
Psi_F=np.conj(np.fft.fft(np.eye(len(t_k)))/np.sqrt(n_t))
# Check Orthonormality
Ones=np.real(np.diag(np.dot(np.conj(Psi_F),Psi_F)))
# Fourier transform of the input
U_DFT=np.dot(np.conj(Psi_F),u)
# Compare this result with FFT
plt.subplot(2,1,1)
plt.plot(np.angle(U),np.angle(U_DFT))
plt.subplot(2,1,2)
plt.plot(np.abs(U),np.abs(U_DFT))


# Fourier transform of the output
Y_DFT=np.dot(H_d,U_DFT)
# Inverse transform
y_Filt3=np.dot(Psi_F,Y_DFT)

# Compare the results
plt.plot(t_k,y_Filt)
plt.plot(t_k,y_Filt2,'ko')
plt.plot(t_k,y_Filt3)

# This means that the matrix C can also be obtained as follow

C_CHECK=np.linalg.multi_dot([(Psi_F),H_d,np.conj(Psi_F)])*np.sqrt(n_t)  

# Compare results
plt.plot(C_CHECK[0,:])
plt.plot(C_H[0,:])




