# -*- coding: utf-8 -*-
"""
Created on Thu Feb  6 13:49:50 2020

@author: mendez
"""

## Exercise 2: Eigenfunction vs DFT vs POD vs DMD

import numpy as np # Numpy is for scientific computing
import matplotlib.pyplot as plt  # Matplotlib is for plotting.
from numpy import linalg as LA

# 1 . Data Preparation
W=10; Pa=60  #Parameters
n = np.arange(1,21,1);N=len(n) # Summation index
# --------------------- Time Discretization------------------
n_t=200; f_s=10; dt=1/f_s; t=np.arange(0,n_t)*dt;
# ------------------- Space Discretization------------------
n_y=2000; y=np.linspace(-1,1,n_y); dy=y[2]-y[1]  
# 2. Initialize Matrices:
Phi_En=np.zeros((n_y,N)); #Initialize Spatial Basis
Sigma_En=np.zeros((N,N)); #Initialize Amplitude Matrix
Psi_En=np.zeros((n_t,N)); #Initialize Temporal Basis
# 3. Construct the Spatial Eigenfunction basis
for i in n: 
 N_o=2*i-1 # Odd numbers  
 # Spatial Structures 
 Phi_En[:,i-1]=np.cos(N_o*np.pi*y/2) 
 # Amplitudes
 Sigma_En[i-1,i-1]=(16*Pa)/\
 (N_o*np.pi*np.sqrt((2*W)**4+N_o**4*np.pi**4)) 
 # Temporal Structures
 Psi_En[:,i-1]=(-1)**i*np.cos\
 (t-np.arctan((4*W**2)/(N_o**2*np.pi**2))) 
    
   
## Test: Orthogonality
#TEST=np.dot(np.transpose(T_n),T_n)   
#TEST=np.dot(np.transpose(Y_n),Y_n)   
   
   
# Assembly eigenfunction solution
U_A=LA.multi_dot([Phi_En,Sigma_En,np.transpose(Psi_En)])   
# Compute mean flow
U_M=0.5*(1-y**2) # Mean Velocity Profile
# Add the mean flow
D_full=np.transpose(np.tile(U_M,(n_t,1)))+U_A; D=U_A

# Plot 5 time steps
fig, ax = plt.subplots(figsize=(4, 3)) # This creates the figure
# Or you can plot it as streamlines
for k in range(10,15,1):
 Index=1+k*10   
 plt.plot(y,D[:,Index],'k')
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
#ax.set_aspect('equal') # Set equal aspect ratio
plt.xlabel('$\hat{y}$',fontsize=14)
plt.ylabel('$\hat{u}$',fontsize=14)
plt.xlim([-1,1])
plt.tight_layout()
Name='Example_Profiles.pdf'
plt.savefig(Name, dpi=200) 

def Modal_Analysis_2(D,Psi,Sort):  
    """
    This function implements algorithm 2 in the notes.          
    :param D: Dataset Matrix
    :param PSI: Temporal Basis
    :param Sort: 1 for sorting 0 for not
    """  

    # Step 1: Prepare Projection
    P_Psi_s=np.dot(np.transpose(np.conjugate(Psi)),Psi)
    P_Psi=np.dot(np.conjugate(Psi),LA.inv(P_Psi_s))
    # Step 2: Project the data (line 1)
    D_psi=np.dot(D,P_Psi)           
    # Step 3: Loop for Phi and Sigma
    # Initialize output first
    R=Psi.shape[1] 
    Phi=np.zeros((D.shape[0],R),dtype=complex)
    Sigma=np.zeros(R,)     
    # Loop for normalization (line 3)
    for r in range(0,R):
     Sigma[r]=LA.norm(D_psi[:,r])
     Phi[:,r]=D_psi[:,r]/Sigma[r]  
     print('Computing Mode '+str(r+1)+'/'+str(R)) 
    # We add also the possibility to do sorting
    if Sort:
     # find indices for sorting in decreasing order   
     Indices=np.flipud(np.argsort(Sigma)) 
     # Sort The decomposition
     Sigma=Sigma[Indices] 
     Phi=Phi[:,Indices] 
     Psi=Psi[:,Indices] 
    return Phi, Sigma, Psi

######## Decomposition 1: DFT ###########
Psi_F=np.conj(np.fft.fft(np.eye(n_t)))/np.sqrt(n_t)     
# DFT
Phi_F, Sigma_F,Psi_F=Modal_Analysis_2(D,Psi_F,False) 
# Check error
D_F=LA.multi_dot([Phi_F,np.diag(Sigma_F),\
                  np.transpose(Psi_F)]) 
Error=LA.norm(D-D_F)/np.linalg.norm(D)
print('Convergence Error: E_F='+"{:.2f}".format(Error/100)+' %')


# For later convenience, prepare vector of digital frequencies
Freqs=np.fft.fftfreq(n_t)*1/dt
# The DFT spectra reads
plt.plot(np.fft.fftshift(Freqs),np.fft.fftshift(Sigma_F),'ko-')
IND=np.argmax(Sigma_F)
ff=Freqs[IND]


######## Decomposition 2: POD ###########
# Construct K
K = np.dot(D.transpose(), D) 
# Comput the Temporal basis Psi_P
Psi_P, Lambda_P, _ = np.linalg.svd(K)
# Compute POD
Phi_P, Sigma_P,Psi_P=Modal_Analysis_2(D,Psi_P,False) 
# Check error
D_P=LA.multi_dot([Phi_P,np.diag(Sigma_P),\
                  np.transpose(Psi_P)]) 
Error=LA.norm(D-D_P)/np.linalg.norm(D)
print('Convergence Error: E_P='+"{:.2f}".format(Error/100)+' %')


######## Decomposition 4: DMD (SVD based) ###########
# Prepare the two portions of data
D_1=D[:,0:n_t-1] ;D_2=D[:,1:n_t]
# Compute svd of the first Part
U, S, VH = LA.svd(D_1); V=np.transpose(np.conj(VH)); 
# Take the leading modes
r_t=LA.matrix_rank(D_1)
Uc=U[:,0:r_t]; Sc=S[0:r_t]; Vc=V[:,0:r_t]
# Compute approximated propagator
P_A=LA.multi_dot([np.transpose(Uc),D_2,\
                  Vc,np.linalg.inv(np.diag(Sc))])
# Compute eigenvalues of the system
Lambda, V = LA.eig(P_A)
# Freqs
F_r=np.imag(np.log(Lambda)/(2*np.pi*dt))

# Assembly Temporal Structure
Psi_D=np.transpose\
(np.vander(Lambda,n_t,\
           increasing=True))/np.sqrt(n_t)
# Compute DMD
Phi_D, Sigma_D,Psi_D=Modal_Analysis_2(D,Psi_D,False) 
# Check error
D_D=LA.multi_dot([Phi_D,np.diag(Sigma_D),\
                  np.transpose(Psi_D)]) 
Error=LA.norm(D-D_D)/np.linalg.norm(D)
print('Convergence Error: E_D='+"{:.2f}".format(Error/100)+' %')



# Plot 5 time steps
fig, ax = plt.subplots(figsize=(4, 3)) # This creates the figure
# Or you can plot it as streamlines
for k in range(10,15,1):
 Index=1+k*10    
 plt.plot(y,D[:,Index],'k')
 plt.plot(y,D_F[:,Index],'r')
 plt.plot(y,D_P[:,Index],'b')
 plt.plot(y,D_D[:,Index],'g')
 
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
#ax.set_aspect('equal') # Set equal aspect ratio
plt.xlabel('$\hat{y}$',fontsize=14)
plt.ylabel('$\hat{u}$',fontsize=14)
plt.tight_layout()

# Sort DFT 
Indices=np.flipud(np.argsort(Sigma_F)) # find indices for sorting in decreasing order
Sigma_F_S=Sigma_F[Indices]
Phi_F_S=Phi_F[:,Indices] # Sorted Spatial Structures Matrix
Psi_F_S=Psi_F[:,Indices] # Sorted Temporal Structures Matrix
# Sort DMD
Indices=np.flipud(np.argsort(Sigma_D)) # find indices for sorting in decreasing order
Sigma_D_S=Sigma_D[Indices]
Phi_D_S=Phi_D[:,Indices] # Sorted Spatial Structures Matrix
Psi_D_S=Psi_D[:,Indices] # Sorted Temporal Structures Matrix



## Plot to conclude the exercise
fig, ax = plt.subplots(figsize=(5, 3)) # This creates the figure
plt.plot(np.arange(1,len(Sigma_En)+1),np.diag(Sigma_En),'ko',label='$\mathcal{E}$')
plt.plot(np.arange(1,len(Sigma_P)+1),(Sigma_P)/np.sqrt(n_t*n_y),'rs',label='$\mathcal{P}$')
plt.plot(np.arange(1,len(Sigma_F)+1),(Sigma_F_S)/np.sqrt(n_t*n_y),'bv',label='$\mathcal{F}$')
plt.plot(np.arange(1,len(Sigma_D)+1),(Sigma_D_S)/np.sqrt(n_t*n_y),'gd',label='$\mathcal{D}$')
plt.xlim([0.5,10])
plt.ylim([0.01,2])
plt.xlabel('$r$',fontsize=14)
plt.ylabel('$\sigma_r$',fontsize=14)
ax.legend(fontsize=12)
plt.yscale('log')
plt.tight_layout()
Name='Amplitudes_EX.pdf'
plt.savefig(Name, dpi=200) 


## Figure Modes EIGEN F--------------------------------
## Plot to conclude the exercise
fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(y,Phi_En[:,0]/LA.norm(Phi_En[:,0]),'k',label='$1$')
plt.plot(y,Phi_En[:,1]/LA.norm(Phi_En[:,1]),'b--',label='$2$')
plt.plot(y,Phi_En[:,2]/LA.norm(Phi_En[:,2]),'r-.',label='$3$')
plt.xlim([-1,1])
plt.ylim([-0.047,0.035])
plt.title('$\phi_{\mathcal{E}1},\
          \phi_{\mathcal{E}2},\
          \phi_{\mathcal{E}3}$',fontsize=16)
plt.legend(loc="lower left", mode = "expand", ncol = 3,fontsize=16)
plt.xlabel('$\hat{y}$',fontsize=14)
plt.tight_layout()
Name='Phi_E.pdf'
plt.savefig(Name, dpi=200) 

fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(t,Psi_En[:,0]/LA.norm(Psi_En[:,0]),'k',label='$1$')
plt.plot(t,Psi_En[:,1]/LA.norm(Psi_En[:,1]),'b--',label='$2$')
plt.plot(t,Psi_En[:,2]/LA.norm(Psi_En[:,2]),'r-.',label='$3$')
plt.xlim([0,20])
plt.ylim([-0.12,0.12])
plt.xlabel('$\hat{t}$',fontsize=14)
plt.title('$\psi_{\mathcal{E}1},\
          \psi_{\mathcal{E}2}$',fontsize=16)
plt.tight_layout()
Name='Psi_E.pdf'
plt.savefig(Name, dpi=200) 


## Figure Modes  POD--------------------------------
## Plot to conclude the exercise
fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(y,Phi_P[:,0],'k',label='$1$')
plt.plot(y,Phi_P[:,1],'b--',label='$2$')
plt.xlim([-1,1])
plt.ylim([-0.035,0.06])
plt.title('$\phi_{\mathcal{P}1},\
            \phi_{\mathcal{P}2}$',fontsize=16)
plt.legend(loc="lower left", mode = "expand", ncol = 3,fontsize=16)
plt.xlabel('$\hat{y}$',fontsize=14)
plt.tight_layout()
Name='Phi_P.pdf'
plt.savefig(Name, dpi=200) 

fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(t,Psi_P[:,0]/LA.norm(Psi_P[:,0]),'k',label='$1$')
plt.plot(t,Psi_P[:,1]/LA.norm(Psi_P[:,1]),'b--',label='$2$')
plt.xlim([0,20])
plt.ylim([-0.12,0.12])
plt.xlabel('$\hat{t}$',fontsize=14)
plt.title('$\psi_{\mathcal{P}1},\
            \psi_{\mathcal{P}2}$',fontsize=16)
plt.tight_layout()
Name='Psi_P.pdf'
plt.savefig(Name, dpi=200) 


## Figure Modes  DFT--------------------------------
## Plot to conclude the exercise
fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(y,np.real(Phi_F_S[:,0]),'k',label='$R1$')
plt.plot(y,np.imag(Phi_F_S[:,0]),'b--',label='$I1$')
plt.plot(y,np.imag(Phi_F_S[:,1]),'r-.',label='$I2$')
plt.xlim([-1,1])
plt.ylim([-0.038,0.028])
plt.title('$\phi_{\mathcal{F}1},\
            Re\{\phi_{\mathcal{F}2}\},\
            Im\{\phi_{\mathcal{F}2}\}$',fontsize=16)
plt.legend(loc="lower left", mode = "expand", ncol = 3,fontsize=16)
plt.xlabel('$\hat{y}$',fontsize=14)
plt.tight_layout()
Name='Phi_F.pdf'
plt.savefig(Name, dpi=200) 


fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(t,np.real(Psi_F_S[:,0])/LA.norm(np.real(Psi_D_S[:,0])),'k',label='$R1$')
plt.plot(t,np.imag(Psi_F_S[:,0])/LA.norm(np.imag(Psi_D_S[:,0])),'b--',label='$I1$')
plt.plot(t,np.imag(Psi_F_S[:,1])/LA.norm(np.imag(Psi_D_S[:,1])),'r--',label='$I2$')
plt.xlim([0,20])
plt.ylim([-0.12,0.12])
plt.xlabel('$\hat{t}$',fontsize=14)
plt.title('$Re\{\psi_{\mathcal{F}1}\},\
            Im\{\psi_{\mathcal{F}2}\},\
            Im\{\psi_{\mathcal{F}2}\}$',fontsize=16)
plt.tight_layout()
Name='Psi_F.pdf'
plt.savefig(Name, dpi=200) 


## Figure Modes  DMD--------------------------------
## Plot to conclude the exercise
fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(y,np.real(Phi_D_S[:,0])/LA.norm((Psi_D_S[:,0])),'k',label='$R1$')
plt.plot(y,np.imag(Phi_D_S[:,0])/LA.norm((Psi_D_S[:,0])),'b--',label='$I1$')
plt.plot(y,np.imag(Phi_D_S[:,1])/LA.norm((Psi_D_S[:,1])),'r-.',label='$I2$')
plt.xlim([-1,1])
plt.ylim([-0.038,0.028])
plt.title('$\phi_{\mathcal{D}1},\
            Im\{\phi_{\mathcal{D}1}\},\
            Im\{\phi_{\mathcal{D}2}\}$',fontsize=16)
plt.legend(loc="lower left", mode = "expand", ncol = 3,fontsize=16)
plt.xlabel('$\hat{y}$',fontsize=14)
plt.tight_layout()
Name='Phi_D.pdf'
plt.savefig(Name, dpi=200) 


fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(t,np.real(Psi_D_S[:,0])/LA.norm((Psi_D_S[:,0])),'k',label='$R1$')
plt.plot(t,np.imag(Psi_D_S[:,0])/LA.norm((Psi_D_S[:,0])),'b--',label='$I1$')
plt.plot(t,np.imag(Psi_D_S[:,1])/LA.norm((Psi_D_S[:,1])),'r--',label='$I2$')
plt.xlim([0,20])
plt.ylim([-0.12,0.12])
plt.xlabel('$\hat{t}$',fontsize=14)
plt.title('$\psi_{\mathcal{D}1},\
            Re\{\psi_{\mathcal{D}2}\},\
            Im\{\psi_{\mathcal{D}2}\}$',fontsize=16)
plt.tight_layout()
Name='Psi_D.pdf'
plt.savefig(Name, dpi=200) 

plt.close(fig='all')
