# -*- coding: utf-8 -*-
"""
Created on Mon Dec 30 15:29:55 2019

This is the first version of the Ex 1- Same structure as in the Matlab exercise.
@author: mendez
"""

import numpy as np
import matplotlib.pyplot as plt
from scipy import signal
import shutil # To copy the figures in the Latex folder

from numpy import linalg as LA


# %% Set Up and Create Data Matrix D

# 1 . Data Preparation
dt =0.5; t =np.arange(0,620+dt,dt); n_t=len(t); f_s=1/dt #Time Discretization
dy =0.01; y=np.arange(-1,1+dy,dy); n_y=len(y); #Space Discretization

# Set the parameters of the two pressure pulsation
W_1=4      # Womersley 1
Pa_1=20    # P Amplitude 1
W_2=1      # Womersley 2
Pa_2=2     # P Amplitude 2
u_A_r_1=np.zeros((n_y,n_t)) # Solution W_1
u_A_r_2=np.zeros((n_y,n_t)) # Solution W_2

# Add a step-like growth of the large scale mode 
# and a gaussian window to the fast scale
T_A_F=max(t)/2 # Location of the window center
STEP_S=np.ones(np.shape(t)) # Initialize the step
STEP_S[0:int(np.floor(n_t/10)):1]=0
STEP_S[int(len(STEP_S)-np.floor(n_t/10)):len(STEP_S):1]=0 # Step Definition
#Smooth the step like kernel
N_M=50;  
h=np.ones((N_M))/N_M # Create impulse response moving average

# We construct a smoothed step repeading 5 times a median filter
for i in range(0,4):
  STEP_S=np.convolve(STEP_S,h,'same')
  
# Create also the second kind of smoothed STEP
STEP_F=np.exp(-(t-T_A_F)**2/5000);  
# Check them
#plt.plot(t,STEP_S) 
#plt.plot(t,STEP_F)


  
  
 # 2. Initialize Matrices:
n = np.arange(0,20,1) 
N=len(n) #Modes to include in summation :
Y_n1=np.zeros((n_y,N)) #Initialize Spatial Basis
A_n1=np.zeros((N,N)) #Initialize Amplitude Matrix
T_n1=np.zeros((n_t,N)) #Initialize Temporal Basis
U_A1=np.zeros((n_t,n_y)) #Initialize PDE solution
 
Y_n2=np.zeros((n_y,N)) #Initialize Spatial Basis
A_n2=np.zeros((N,N)) #Initialize Amplitude Matrix
T_n2=np.zeros((n_t,N)) #Initialize Temporal Basis
U_A2=np.zeros((n_t,n_y)) #Initialize PDE solution
    
# 3. Construct Spatial Eigenfunction basis (Fast Scale)
ns=np.zeros(n.shape)
for i in range(0,20):     
  N=2*i+1; # odd number in the series
  ns[i]=N
  # Pulsation Mode 1
  Y_n1[:,i]=np.cos(N*np.pi*y/2) # Spatial Eigenfunction basis
  A_n1[i,i] =(16*Pa_1)/(N*np.pi*np.sqrt((2*W_1)**4+N**4*np.pi**4)); # Amplitudes 
  T_n1[:,i]=(-1)**(i+1)*np.sin(t)*STEP_F# # Temporal coefficients
  # Pulsation Mode 2
  Y_n2[:,i]=np.cos(N*np.pi*y/2); # Spatial Eigenfunction basis
  A_n2[i,i]=(16*Pa_2)/(N*np.pi*np.sqrt((2*W_2)**4+N**4*np.pi**4)); # Amplitudes 
  T_n2[:,i]=(-1)**(i+1)*np.sin((W_2/W_1)**2*t)*STEP_S; # Temporal coefficients
     

 # Assembly the response to each pulsation 
u_A_r_1=np.linalg.multi_dot([Y_n1,A_n1,np.transpose(T_n1)])
u_A_r_2=np.linalg.multi_dot([Y_n2,A_n2,np.transpose(T_n2)])
  
# Observe the time evolution in the centerline for each
fig, ax = plt.subplots(figsize=(6, 3)) # This creates the figure
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(t,u_A_r_2[int(np.floor(n_y/2)),:],'r-.',label='$F_1$')
plt.plot(t,u_A_r_1[int(np.floor(n_y/2)),:],'k-',label='$F_2$')
ax.legend(fontsize=12)
plt.xlabel('$\hat{t}$',fontsize=13)
plt.ylabel('$\hat{u}(\hat{y}=0,t)$',fontsize=13)
#plt.title('Centerline Vel Evolution',fontsize=13)
plt.xlim([0,600])
plt.tight_layout(pad=0.6, w_pad=0.3, h_pad=0.8)
pos1 = ax.get_position() # get the original position 
pos2 = [pos1.x0 + 0.01, pos1.y0 + 0.01,  pos1.width *0.95, pos1.height *0.95] 
ax.set_position(pos2) # set a new position
Name='Centerline_Evolution.pdf'
plt.savefig(Name, dpi=200) 
shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory


#Assembly the Data Matrix for the test Case
#u_Mb=0.5*(1-y**2) #Compute the mean Flow
#u_M=np.transpose(np.tile(u_Mb,(n_t,1))) #Repeat mean to obtain a matrix
D=u_A_r_1+u_A_r_2 #Complete analytical Solution (also adding back the mean)


# %% We first comput the POD of this dataset
# Compute K
K = np.dot(D.transpose(), D)
# Compute Temporal Structures and Amplitudes
Psi_P, Lambda_P, _ = np.linalg.svd(K); 
# Complete Decomposition
from Functions_Chap_7 import Modal_Analysis_2
Phi_P, Sigma_P,Psi_P=Modal_Analysis_2(D,Psi_P,False)


# Plot the POD results
## Figure Modes  POD--------------------------------
## Plot to conclude the exercise
fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(y,Phi_P[:,0],'k',label='$1$')
plt.plot(y,Phi_P[:,1],'b--',label='$2$')
plt.xlim([-1,1])
plt.ylim([-0.12,0.13])
plt.title('$\phi_{\mathcal{P}1},\
            \phi_{\mathcal{P}2}$',fontsize=16)
plt.legend(loc="lower left", mode = "expand", ncol = 3,fontsize=16)
plt.xlabel('$\hat{y}$',fontsize=14)
plt.tight_layout()
Name='Phi_P_Ex3.pdf'
plt.savefig(Name, dpi=200) 
shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory


fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(t,Psi_P[:,0]/LA.norm(Psi_P[:,0]),'k',label='$1$')
plt.plot(t,Psi_P[:,1]/LA.norm(Psi_P[:,1]),'b--',label='$2$')
plt.xlim([0,600])
plt.ylim([-0.12,0.12])
plt.xlabel('$\hat{t}$',fontsize=14)
plt.title('$\psi_{\mathcal{P}1},\
            \psi_{\mathcal{P}2}$',fontsize=16)
plt.tight_layout()
Name='Psi_P_Ex3.pdf'
plt.savefig(Name, dpi=200) 
shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory


####### Check the diagonal of K_hat so see what filter is needed
Freq=np.fft.fftfreq(int(n_t))*f_s # Frequency Axis
Freq_s=np.fft.fftshift(Freq)
# Compute the 2D FFT of K
K_HAT=np.fft.fft2(K)/n_t;
# The K_F is
# Create Fourier Matrix
Psi_F=np.conj(np.fft.fft(np.eye(n_t)))/np.sqrt(n_t)
# Compute cross spectral density
K_F=LA.multi_dot([Psi_F,K,np.conj(Psi_F)])
# This is equivalent to 
K_F2=LA.multi_dot([Psi_F,Psi_F,K_HAT]); Check=K_F-K_F2


fig, ax = plt.subplots(figsize=(4,4))
#ax.set_xlim([-0.5,0.5])
#ax.set_ylim([-0.5,0.5])
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=12)
plt.rc('ytick',labelsize=12)
plt.pcolor(Freq_s,Freq_s,np.fft.fftshift(np.abs(K_HAT)/n_y)) # We normalize the result
ax.set_aspect('equal') # Set equal aspect ratio
ax.set_xlabel('$\hat{f}[-]$',fontsize=14)
ax.set_ylabel('$\hat{f}[-]$',fontsize=14)
ax.set_xticks(np.arange(-1,1.1,0.25))
ax.set_yticks(np.arange(-1,1.1,0.25))
ax.set_xlim(-0.25,0.25)
ax.set_ylim(-0.25,0.25)
plt.tight_layout(pad=1, w_pad=0.5, h_pad=1.0)
plt.clim(0,10) # From here  downward is just for plotting purposes
plt.tight_layout()
Name='Correlation_Spectra_Ex3.png'
plt.savefig(Name, dpi=200)  
shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory


plt.close(fig='all')
fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(Freq_s,np.fft.fftshift(np.abs(np.diag(K_F)))/n_y,'k')
plt.xlim([0,0.35])
#plt.ylim([-0.12,0.12])
plt.xlabel('$\hat{f}$',fontsize=14)
plt.title('$|diag(\mathbf{K}_{\mathcal{F}}))|$',fontsize=16)
plt.tight_layout()
Name='diag_K_F.pdf'
plt.savefig(Name, dpi=200) 
shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory

# %% Filter the data

## The impulse response of a filter that keeps only the finest scale is
N=511# Filter order
# High pass filter (f_c=0.1)
h_HP=signal.firwin(N, 0.1, \
                   pass_zero=False,window='hamming', fs=f_s)

######## We perform the filter in three ways

# Method 1 Use Convolutions
from scipy.linalg import circulant
# Construct circulant matrix
h_HPe=np.zeros(n_t); h_HPe[0:N]=h_HP 
C_H1=circulant(h_HPe);C_H2=circulant(np.flip(h_HPe))
# Use equation 36 twice
D_FILT_1=np.dot(np.dot(D,np.transpose(C_H1)),np.transpose(C_H2))

# Method 2 In the frequncy domain
# Frequency Response from h
H_hat=np.fft.fft(h_HP,n_t)
# Zero-Phase (anticausal) Response 
H_hat_zs=H_hat*np.conj(H_hat)
# Diagonal Matrix , eq.35 (left)
H_diag=np.diag(H_hat_zs)
# Filtered data 
D_FILT_2=LA.multi_dot([D,np.conj(Psi_F),H_diag,Psi_F])

# Plot velocity in the center
plt.plot(close='all')
plt.plot(t,D_FILT_1[int(np.floor(n_y/2)),:],'r-.')
plt.plot(t,D_FILT_2[int(np.floor(n_y/2)),:],'ko')
plt.plot(t,D[int(np.floor(n_y/2)),:],'bo')


# Correlation of filtered portion
K_filt1=np.dot(np.transpose(D_FILT_2),D_FILT_2)

# 
# %% Compute the POD of the filtered data and filtered correlation

# We filter the correlation matrix.
######## Method 1 from Definition.
## Construct H calligraphic
H_hat_r=np.array([H_hat_zs])
H_cal=np.dot(np.ones((n_y,1)),H_hat_r)/np.sqrt(n_y)
H_underline=np.dot(np.conj(np.transpose(H_cal)),H_cal)
K_filt_F=K_F*H_underline
#
#plt.plot(abs(np.diag(K_F)))
#plt.plot(abs(np.diag(K_filt_F)))

K_filt2=LA.multi_dot([np.conj(Psi_F),K_filt_F,Psi_F])
# Compute diagonal

#
#plt.plot(np.diag(K_filt1))
#plt.plot(np.diag(K_filt2))


# Method 3: from K_HAT
K_HAT=np.fft.fft2(K)/n_t;
K_hat_filt=K_HAT*H_underline
K_filt3=LA.multi_dot([Psi_F,K_hat_filt,Psi_F])


# Method 4
K_filt4=np.real(np.fft.ifft2(K_HAT*H_underline))*n_t




plt.plot(np.diag(K_filt1))
plt.plot(np.diag(K_filt2))
plt.plot(np.diag(K_filt3))
plt.plot(np.diag(K_filt4))


# %% POD of the filtered data 
# Compute K
# Compute Temporal Structures and Amplitudes
Psi_P, Lambda_P, _ = np.linalg.svd(K_filt3); 
# Complete Decomposition
from Functions_Chap_7 import Modal_Analysis_2
Phi_P, Sigma_P,Psi_P=Modal_Analysis_2(D,Psi_P,False)


# Plot the POD results of  the filtered data
## Figure Modes  POD--------------------------------
## Plot to conclude the exercise
fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(y,Phi_P[:,0],'k')
plt.xlim([-1,1])
plt.ylim([-0.12,0.13])
plt.title('$\phi_{\mathcal{P}1}$',fontsize=16)
#plt.legend(loc="lower left", mode = "expand", ncol = 3,fontsize=16)
plt.xlabel('$\hat{y}$',fontsize=14)
plt.tight_layout()
Name='Phi_P_Ex3_filt.pdf'
plt.savefig(Name, dpi=200) 
shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory


fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(t,Psi_P[:,0]/LA.norm(Psi_P[:,0]),'k')
plt.xlim([0,600])
plt.ylim([-0.12,0.12])
plt.xlabel('$\hat{t}$',fontsize=14)
plt.title('$\psi_{\mathcal{P}1}$',fontsize=16)
plt.tight_layout()
Name='Psi_P_Ex3_filt.pdf'
plt.savefig(Name, dpi=200) 
shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory




## Test the DMD

######## Decomposition 4: DMD (SVD based) ###########
# Prepare the two portions of data
D_1=D[:,0:n_t-1] ;D_2=D[:,1:n_t]
# Compute svd of the first Part
U, S, VH = LA.svd(D_1); V=np.transpose(np.conj(VH)); 
# Take the leading modes
r_t=LA.matrix_rank(D_1)
Uc=U[:,0:r_t]; Sc=S[0:r_t]; Vc=V[:,0:r_t]
# Compute approximated propagator
P_A=LA.multi_dot([np.transpose(Uc),D_2,\
                  Vc,np.linalg.inv(np.diag(Sc))])
# Compute eigenvalues of the system
Lambda, V = LA.eig(P_A)
# Freqs
F_r=np.imag(np.log(Lambda)/(2*np.pi*dt))

# Assembly Temporal Structure
Psi_D=np.transpose\
(np.vander(Lambda,n_t,\
           increasing=True))/np.sqrt(n_t)
# Compute DMD
Phi_D, Sigma_D,Psi_D=Modal_Analysis_2(D,Psi_D,False) 
# Check error
D_D=LA.multi_dot([Phi_D,np.diag(Sigma_D),\
                  np.transpose(Psi_D)]) 
Error=LA.norm(D-D_D)/np.linalg.norm(D)
print('Convergence Error: E_D='+"{:.2f}".format(Error/100)+' %')









    