# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 15:03:54 2020

@author: mendez
"""

# We construct our mPOD function and test it on the data from Ex 3

# First, we construct the data (copy paste from Ex 3)


import numpy as np
import matplotlib.pyplot as plt



# %% Set Up and Create Data Matrix D

# 1 . Data Preparation
dt =0.5; t =np.arange(0,620+dt,dt); n_t=len(t); f_s=1/dt #Time Discretization
dy =0.01; y=np.arange(-1,1+dy,dy); n_y=len(y); #Space Discretization

# Set the parameters of the two pressure pulsation
W_1=4      # Womersley 1
Pa_1=20    # P Amplitude 1
W_2=1      # Womersley 2
Pa_2=2     # P Amplitude 2
u_A_r_1=np.zeros((n_y,n_t)) # Solution W_1
u_A_r_2=np.zeros((n_y,n_t)) # Solution W_2

# Add a step-like growth of the large scale mode 
# and a gaussian window to the fast scale
T_A_F=max(t)/2 # Location of the window center
STEP_S=np.ones(np.shape(t)) # Initialize the step
STEP_S[0:int(np.floor(n_t/10)):1]=0
STEP_S[int(len(STEP_S)-np.floor(n_t/10)):len(STEP_S):1]=0 # Step Definition
#Smooth the step like kernel
N_M=50;  
h=np.ones((N_M))/N_M # Create impulse response moving average

# We construct a smoothed step repeading 5 times a median filter
for i in range(0,4):
  STEP_S=np.convolve(STEP_S,h,'same')
  
# Create also the second kind of smoothed STEP
STEP_F=np.exp(-(t-T_A_F)**2/5000);  
  
 # 2. Initialize Matrices:
n = np.arange(0,20,1) 
N=len(n) #Modes to include in summation :
Y_n1=np.zeros((n_y,N)) #Initialize Spatial Basis
A_n1=np.zeros((N,N)) #Initialize Amplitude Matrix
T_n1=np.zeros((n_t,N)) #Initialize Temporal Basis
U_A1=np.zeros((n_t,n_y)) #Initialize PDE solution
 
Y_n2=np.zeros((n_y,N)) #Initialize Spatial Basis
A_n2=np.zeros((N,N)) #Initialize Amplitude Matrix
T_n2=np.zeros((n_t,N)) #Initialize Temporal Basis
U_A2=np.zeros((n_t,n_y)) #Initialize PDE solution
    
# 3. Construct Spatial Eigenfunction basis (Fast Scale)
ns=np.zeros(n.shape)
for i in range(0,20):     
  N=2*i+1; # odd number in the series
  ns[i]=N
  # Pulsation Mode 1
  Y_n1[:,i]=np.cos(N*np.pi*y/2) # Spatial Eigenfunction basis
  A_n1[i,i] =(16*Pa_1)/(N*np.pi*np.sqrt((2*W_1)**4+N**4*np.pi**4)); # Amplitudes 
  T_n1[:,i]=(-1)**(i+1)*np.sin(t)*STEP_F# # Temporal coefficients
  # Pulsation Mode 2
  Y_n2[:,i]=np.cos(N*np.pi*y/2); # Spatial Eigenfunction basis
  A_n2[i,i]=(16*Pa_2)/(N*np.pi*np.sqrt((2*W_2)**4+N**4*np.pi**4)); # Amplitudes 
  T_n2[:,i]=(-1)**(i+1)*np.sin((W_2/W_1)**2*t)*STEP_S; # Temporal coefficients
     

 # Assembly the response to each pulsation 
u_A_r_1=np.linalg.multi_dot([Y_n1,A_n1,np.transpose(T_n1)])
u_A_r_2=np.linalg.multi_dot([Y_n2,A_n2,np.transpose(T_n2)])
  
D=u_A_r_1+u_A_r_2 #Complete analytical Solution (also adding back the mean)





# %% Test The mPOD
Nf=np.array([501]) # Filter Orders
F_V=np.array([0.1]) # Frequency Splitting Vector
from Functions_Chap_7 import mPOD_A3
from Functions_Chap_7 import Modal_Analysis_2
Psi_M,Ks= mPOD_A3(D,F_V,Nf,f_s) # Compute PSI_M
Phi_M, Sigma_M,Psi_M=Modal_Analysis_2(D,Psi_M,True)



Freq=np.fft.fftfreq(int(n_t))*f_s # Frequency Axis
Freq_s=np.fft.fftshift(Freq)
K=np.dot(np.transpose(D),D)
K_HAT=np.fft.fft2(Ks[:,:,0])/n_t;

# Show the spectra of the temporal correlation matrices
fig, ax = plt.subplots(figsize=(4,4))
#ax.set_xlim([-0.5,0.5])
#ax.set_ylim([-0.5,0.5])
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=12)
plt.rc('ytick',labelsize=12)
plt.pcolor(Freq_s,Freq_s,np.fft.fftshift(np.abs(K_HAT)/n_y)) # We normalize the result
ax.set_aspect('equal') # Set equal aspect ratio
ax.set_xlabel('$\hat{f}[-]$',fontsize=14)
ax.set_ylabel('$\hat{f}[-]$',fontsize=14)
ax.set_xticks(np.arange(-1,1.1,0.25))
ax.set_yticks(np.arange(-1,1.1,0.25))
ax.set_xlim(-0.25,0.25)
ax.set_ylim(-0.25,0.25)
plt.title('$\widehat{\mathbf{K}}_1$',fontsize=14)
plt.tight_layout(pad=1, w_pad=0.5, h_pad=1.0)
plt.clim(0,10) # From here  downward is just for plotting purposes
plt.tight_layout()
Name='Correlation_Spectra_Ex4_1.png'
plt.savefig(Name, dpi=200)  

K_HAT=np.fft.fft2(Ks[:,:,1])/n_t;

# Show the spectra of the temporal correlation matrices
fig, ax = plt.subplots(figsize=(4,4))
#ax.set_xlim([-0.5,0.5])
#ax.set_ylim([-0.5,0.5])
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=12)
plt.rc('ytick',labelsize=12)
plt.pcolor(Freq_s,Freq_s,np.fft.fftshift(np.abs(K_HAT)/n_y)) # We normalize the result
ax.set_aspect('equal') # Set equal aspect ratio
ax.set_xlabel('$\hat{f}[-]$',fontsize=14)
ax.set_ylabel('$\hat{f}[-]$',fontsize=14)
ax.set_xticks(np.arange(-1,1.1,0.25))
ax.set_yticks(np.arange(-1,1.1,0.25))
ax.set_xlim(-0.25,0.25)
ax.set_ylim(-0.25,0.25)
plt.title('$\widehat{\mathbf{K}}_2$',fontsize=14)
plt.tight_layout(pad=1, w_pad=0.5, h_pad=1.0)
plt.clim(0,10) # From here  downward is just for plotting purposes
plt.tight_layout()
Name='Correlation_Spectra_Ex4_2.png'
plt.savefig(Name, dpi=200)  






## Figure Modes  mPOD--------------------------------
## Plot to conclude the exercise
fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.plot(y,Phi_M[:,0],'k',label='$1$')
plt.plot(y,Phi_M[:,1],'b--',label='$2$')
plt.xlim([-1,1])
plt.ylim([-0.12,0.13])
plt.title('$\phi_{\mathcal{M}1},\
            \phi_{\mathcal{M}2}$',fontsize=16)
plt.legend(loc="upper left", mode = "expand", ncol = 3,fontsize=16)
plt.xlabel('$\hat{y}$',fontsize=14)
plt.tight_layout()
Name='Phi_M_Ex4.pdf'
plt.savefig(Name, dpi=200) 


fig, ax = plt.subplots(figsize=(4.5, 3)) # This creates the figure
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.plot(t,Psi_M[:,0],'k',label='$1$')
plt.plot(t,Psi_M[:,1],'b--',label='$2$')
plt.xlim([0,600])
plt.ylim([-0.12,0.12])
plt.xlabel('$\hat{t}$',fontsize=14)
plt.title('$\psi_{\mathcal{M}1},\
            \psi_{\mathcal{M}2}$',fontsize=16)
plt.tight_layout()
Name='Psi_M_Ex4.pdf'
plt.savefig(Name, dpi=200) 



