# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 17:43:40 2020

@author: mendez
"""


import numpy as np
import matplotlib.pyplot as plt
from Functions_Chap_7 import Plot_Field_TEXT_Jet
from Functions_Chap_7 import Plot_Field_Jet
import os 

import shutil # To copy the figures in the Latex folder



import urllib.request
print('Downloading Data for Tutorial 1...')
url = 'https://osf.io/c28de/download'
urllib.request.urlretrieve(url, 'Ex_4_TR_PIV_Jet.zip')
print('Download Completed! I prepare data Folder')
# Unzip the file 
from zipfile import ZipFile
String='Ex_4_TR_PIV_Jet.zip'
zf = ZipFile(String,'r'); zf.extractall('./'); zf.close()


FOLDER='Ex_4_TR_PIV_Jet'
# Construct Time discretization
n_t=2000; Fs=2000; dt=1/Fs 
t=np.linspace(0,dt*(n_t-1),n_t) # prepare the time axis# 

# Read file number 10 (Check the string construction)
Name=FOLDER+os.sep+'Res%05d'%10+'.dat' # Check it out: print(Name)
n_s, Xg, Yg, Vxg, Vyg, X_S,Y_S=Plot_Field_TEXT_Jet(Name) 
Name='Snapshot_JET.png'
plt.savefig(Name, dpi=200) 

shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory

nxny=int(n_s/2) 
####################### 1. CONSTRUCT THE DATA MATRIX D #################
# Initialize the data matrix D
D=np.zeros([n_s,n_t])
#####################################################################

for k in range(0,n_t):
  Name=FOLDER+os.sep+'Res%05d'%(k+1)+'.dat' # Name of the file to read
  # Read data from a file
  DATA = np.genfromtxt(Name) # Here we have the four colums
  Dat=DATA[1:,:] # Here we remove the first raw, containing the header
  V_X=Dat[:,2]; # U component
  V_Y=Dat[:,3]; # V component
  D[:,k]=np.concatenate([V_X,V_Y],axis=0) # Reshape and assign
  # Obs: the file count starts from 1 but the index must start from 0
  print('Loading Step '+str(k+1)+'/'+str(n_t)) 

## Compute the mPOD
H=4/1000; U0=6.5; 
ST_V=np.array([0.1,0.2,0.5]) # Define scales based on St
F_V=ST_V*U0/H



Nf=np.array([201,201,201]); # This vector collects the length of the filter kernels.


# %% Test The mPOD from the Chapter
from Functions_Chap_7 import mPOD_A3
from Functions_Chap_7 import Modal_Analysis_Ortho
import time
Begin=time.time()
Psi_M,Ks= mPOD_A3(D,F_V,Nf,Fs) # Compute PSI_M
Duration=time.time()-Begin # Measure the time of the projection
print('Decomposition completed in  '+str(Duration)+' seconds')
Phi_M, Sigma_M,Psi_M=Modal_Analysis_Ortho(D,Psi_M,True)

# %% Test the mPOD from Modulo
from mPOD_Functions import mPOD_K
Ex=203 # Extension points for the extrapolation
Keep=np.array([1,1,1]); #These are the band-pass you want to keep (Approximation is always kept).
# Compute the mPOD Temporal Basis
Begin=time.time()
K = np.dot(D.transpose(), D);
PSI_M2,Ks = mPOD_K(K,dt,Nf,Ex,F_V,Keep,'nearest','reduced');
Duration=time.time()-Begin # Measure the time of the projection
print('Decomposition completed in  '+str(Duration)+' seconds')
Phi_M, Sigma_M, Psi_M=Modal_Analysis_Ortho(D,PSI_M2,True)




# Show modes
for j in range(1,6):
 plt.close(fig='all') 
 fig, ax3= plt.subplots(figsize=(5,6))   
 ax=plt.subplot(2,1,1)
 plt.rc('text', usetex=True)    
 plt.rc('font', family='serif')
 plt.rc('xtick',labelsize=12)
 plt.rc('ytick',labelsize=12)
 V_X=Phi_M[0:nxny,j-1]
 V_Y=Phi_M[nxny::,j-1]
 Plot_Field_Jet(X_S,Y_S,V_X,V_Y,True,2,1)
# plt.quiver(X_S,Y_S,V_X,V_Y)
 ax.set_aspect('equal') # Set equal aspect ratio
 ax.set_xticks(np.arange(0,40,10))
 ax.set_yticks(np.arange(10,30,5))
 ax.set_xlim([0,35])
 ax.set_ylim([10,29])
 ax.set_xlabel('$x[mm]$',fontsize=13)
 ax.set_ylabel('$y[mm]$',fontsize=13)
 ax.invert_yaxis() # Invert Axis for plotting purpose
 String_y='$\phi_{\mathcal{M}'+str(j)+'}$'
 plt.title(String_y,fontsize=18)
 plt.tight_layout(pad=1, w_pad=0.5, h_pad=1.0)
 
 ax=plt.subplot(2,1,2)
 Signal=Psi_M[:,j-1]
 s_h=np.abs((np.fft.fft(Signal-Signal.mean())))
 Freqs=np.fft.fftfreq(int(n_t))*Fs
 plt.plot(Freqs*(4/1000)/6.5,s_h,'-',linewidth=1.5)
 plt.xlim(0,0.38)    
 plt.xlabel('$St[-]$',fontsize=18)
 String_y='$\widehat{\psi}_{\mathcal{M}'+str(j)+'}$'
 plt.ylabel(String_y,fontsize=18)
 plt.tight_layout(pad=1, w_pad=0.5, h_pad=1.0)
 Name='mPOD_Mode_'+str(j)+'.png'
 print(Name+' Saved')
 plt.savefig(Name, dpi=300) 


