# -*- coding: utf-8 -*-
"""
Created on Mon Feb 10 17:20:50 2020

@author: mendez
"""



import numpy as np
import matplotlib.pyplot as plt
from Functions_Chap_7 import Plot_Field_TEXT_Cylinder
from Functions_Chap_7 import Plot_Field_Cylinder


import shutil # To copy the figures in the Latex folder



import urllib.request
print('Downloading Data for Tutorial 2...')
url = 'https://osf.io/qa8d4/download'
urllib.request.urlretrieve(url, 'Ex_5_TR_PIV_Cylinder.zip')
print('Download Completed! I prepare data Folder')
# Unzip the file 
from zipfile import ZipFile
String='Ex_5_TR_PIV_Cylinder.zip'
zf = ZipFile(String,'r'); zf.extractall('./'); zf.close()


FOLDER='Ex_5_TR_PIV_Cylinder'
# Construct Time discretization
n_t=13200; Fs=3000; dt=1/Fs 
t=np.linspace(0,dt*(n_t-1),n_t) # prepare the time axis# 


# Read file number 10 (Check the string construction)
Name=FOLDER+os.sep+'Res%05d'%10+'.dat' # Check it out: print(Name)
Name_Mesh=FOLDER+os.sep+'MESH.dat'
n_s, Xg, Yg, Vxg, Vyg, X_S,Y_S=Plot_Field_TEXT_Cylinder(Name,Name_Mesh) 
Name='Snapshot_Cylinder.png'
plt.savefig(Name, dpi=200) 


shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory

nxny=int(n_s/2) 
####################### 1. CONSTRUCT THE DATA MATRIX D #################
# Initialize the data matrix D
D=np.zeros([n_s,n_t])
#####################################################################
PROBE=np.zeros(n_t)


for k in range(0,n_t):
  Name=FOLDER+os.sep+'Res%05d'%(k+1)+'.dat' # Name of the file to read
  # Read data from a file
  DATA = np.genfromtxt(Name,usecols=np.arange(0,2),max_rows=nxny+1) # Here we have the two colums
  Dat=DATA[1:,:] # Here we remove the first raw, containing the header
  V_X=Dat[:,0]; # U component
  V_Y=Dat[:,1]; # V component
  D[:,k]=np.concatenate([V_X,V_Y],axis=0) # Reshape and assign
  # Obs: the file count starts from 1 but the index must start from 0
  print('Loading Step '+str(k+1)+'/'+str(n_t)) 
  PROBE[k]=np.sqrt(V_X[4]**2+V_Y[4]**2)
  
  
# Plot velocity in the top corner
fig, ax = plt.subplots(figsize=(6, 3)) # This creates the figure
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(t,PROBE,'k',)
plt.xlabel('$t[s]$',fontsize=16)
plt.ylabel('$U_{\infty}[m/s]$',fontsize=16)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
#plt.title('Centerline Vel Evolution',fontsize=13)
plt.xlim([0,4.56])
plt.tight_layout(pad=0.6, w_pad=0.3, h_pad=0.8)
pos1 = ax.get_position() # get the original position 
pos2 = [pos1.x0 + 0.01, pos1.y0 + 0.01,  pos1.width *0.95, pos1.height *0.95] 
ax.set_position(pos2) # set a new position
Name='U_Infty.pdf'
plt.savefig(Name, dpi=200) 
shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory

  
  
  
  
  
## Create an animation and compute velocity in a probe
#Vector=np.zeros(600)
#II=0
#plt.ioff()
#Fol_Out = 'Gif_Images_temporary'
#if not os.path.exists(Fol_Out):
#    os.mkdir(Fol_Out)
#    
#for k in range(0,12000,10):
# Name=FOLDER+os.sep+'Res%05d'%(k+1)+'.dat' # Check it out: print(Name)
# n_s, Xg, Yg, Vxg, Vyg, X_S,Y_S=Plot_Field_TEXT_Cylinder(Name,Name_Mesh) 
# print('Printing '+str(k)+'/'+str(n_t))
# Vector[II]=np.sqrt(Vxg[1,1]**2+Vyg[1,1]**2) 
# II=+1
# NameOUT=Fol_Out + os.sep + 'Im%03d' % (k) + '.png'
# plt.savefig(NameOUT, dpi=100) 
# plt.close(fig='all')
#     
  

import time
F_V=np.array([10,290,320,430,470])
Nf=np.array([1201,801,801,801,801]); # This vector collects the length of the filter kernels.
Keep=np.array([1,1,1,1,1])
Ex=1203

from mPOD_Functions import mPOD_K
from Functions_Chap_7 import Modal_Analysis_Ortho
Begin=time.time()
K = np.dot(D.transpose(), D);
PSI_M2,Ks = mPOD_K(K,dt,Nf,Ex,F_V,Keep,'nearest','reduced');
Duration=time.time()-Begin # Measure the time of the projection
print('Decomposition completed in  '+str(Duration)+' seconds')
Phi_M, Sigma_M, Psi_M=Modal_Analysis_Ortho(D,PSI_M2,True)

np.savez('Results',Phi_M=Phi_M,Sigma_M=Sigma_M,Psi_M=Psi_M)

data = np.load('Results.npz')
Phi_M=data['Phi_M']
Psi_M=data['Psi_M']
Sigma_M=data['Sigma_M']

## Show mode 1

# Show modes
for j in range(1,6):
 plt.close(fig='all') 
 fig, ax3= plt.subplots(figsize=(5,5))   
 ax=plt.subplot(2,1,1)
 plt.rc('text', usetex=True)    
 plt.rc('font', family='serif')
 plt.rc('xtick',labelsize=12)
 plt.rc('ytick',labelsize=12)
 V_X=Phi_M[0:nxny,j-1]
 V_Y=Phi_M[nxny::,j-1]
 Plot_Field_Cylinder(X_S,Y_S,V_X,V_Y,True,1,2)
# plt.quiver(X_S,Y_S,V_X,V_Y)
 ax.set_aspect('equal') # Set equal aspect ratio
 ax.set_xlabel('$x[mm]$',fontsize=13)
 ax.set_ylabel('$y[mm]$',fontsize=13)
 ax.set_title('Tutorial 2: Cylinder Wake',fontsize=16)
 ax.set_xticks(np.arange(0,70,10))
 ax.set_yticks(np.arange(-10,11,10))
 ax.set_xlim([0,50])
 ax.set_ylim(-10,10)
 circle = plt.Circle((0,0),2.5,fill=True,color='r',edgecolor='k',alpha=0.5)
 plt.gcf().gca().add_artist(circle)
 String_y='$\phi_{\mathcal{M}'+str(j)+'}$'
 plt.title(String_y,fontsize=18)
 plt.tight_layout(pad=1, w_pad=0.5, h_pad=1.0)
 
 ax=plt.subplot(2,1,2)
 Signal=Psi_M[:,j-1] 
 plt.plot(t,Signal,'-',linewidth=1.5)  
 plt.xlabel('$t_k[s]$',fontsize=13)
 String_y='${\psi}_{\mathcal{M}'+str(j)+'}(t_k)$'
 plt.ylabel(String_y,fontsize=18)
 plt.tight_layout(pad=1, w_pad=0.5, h_pad=1.0)
 Name='C_mPOD_Mode_'+str(j)+'.png'
 print(Name+' Saved')
 plt.savefig(Name, dpi=300) 



